package com.jjt.biz.controller;

import com.jjt.biz.domain.AlarmRecord;
import com.jjt.biz.domain.BizModel;
import com.jjt.biz.service.IAlarmRecordService;
import com.jjt.biz.service.IBizModelService;
import com.jjt.biz.vo.BizAccessVO;
import com.jjt.biz.vo.BizSortVO;
import com.jjt.biz.vo.BizTypeVO;
import com.jjt.biz.vo.LevelSortVO;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.utils.StringUtils;
import com.jjt.system.service.ISysDictDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 告警记录Controller
 *
 * @author jjt
 * @date 2024-08-08
 */
@Api(tags = "业务运行问题管理")
@RestController
@RequestMapping("/alarm/record")
public class AlarmRecordController extends BaseController {
    @Resource
    private IAlarmRecordService alarmRecordService;
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    private ISysDictDataService sysDictDataService;
    @Resource
    private IBizModelService bizModelService;

    /**
     * 查询告警记录列表
     */
    @ApiOperation("查询告警记录列表")
    //@PreAuthorize("@ss.hasPermi('alarm:record:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmRecord alarmRecord) {
        startPage();
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        return getDataTable(list);
    }

    /**
     * 查询告警记录列表
     */
    @ApiOperation("统计数据")
    @GetMapping("/lastMonth")
    public AjaxResult calc(AlarmRecord alarmRecord) {
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        return AjaxResult.success();
//        return getDataTable(list);
    }

    @ApiOperation("业务类型统计")
    @GetMapping("/list/bizType")
    public AjaxResult listBizType(@ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
        List<BizTypeVO> list = alarmRecordService.listBizType(start, end);
        return AjaxResult.success(list);
    }

    @ApiOperation("业务组件统计")
    @GetMapping("/list/bizObj")
    public AjaxResult listBizObj(@ApiParam(value = "业务类型") String type, @ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
        List<BizTypeVO> list = alarmRecordService.listBizObj(type, start, end);
        return AjaxResult.success(list);
    }

    @ApiOperation("业务指标统计")
    @GetMapping("/list/getMetrics")
    public AjaxResult getMetrics(@ApiParam(value = "对象ID") Long objId, @ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
        List<BizTypeVO> list = new ArrayList();
        String sql = "SELECT a.obj_metrics_id,b.metrics_name,COUNT(*) num FROM alarm_record a,biz_obj_metrics b WHERE a.obj_metrics_id=b.obj_metrics_id";
        Vector<Object> v = new Vector<>();
        if (objId != null) {
            sql += " AND b.obj_id=?";
            v.add(objId);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }
        sql += "  GROUP by a.obj_metrics_id,b.metrics_name order by num desc";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizTypeVO ls = new BizTypeVO();
            Integer obj_id = (Integer) map.get("obj_metrics_id");
            ls.setId(obj_id + "");
            String obj_name = (String) map.get("metrics_name");
            ls.setName(obj_name);
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

        return AjaxResult.success(list);
    }

    @ApiOperation("应用组件统计")
    @GetMapping("/list/bizSort")
    public AjaxResult listBizSort(@ApiParam(value = "分类") String bizType, @ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
        List<BizSortVO> list = new ArrayList();
//          暂时使用模拟数据
        String sql = "SELECT a.obj_id,b.obj_name,b.obj_type,COUNT(*) num FROM alarm_record a,biz_obj b WHERE a.obj_id=b.obj_id";
        Vector<Object> v = new Vector<>();
        if (StringUtils.isNotEmpty(bizType)) {
            sql += " AND b.obj_type=?";
            v.add(bizType);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }
        sql += " GROUP BY a.obj_id,b.obj_name,b.obj_type";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizSortVO ls = new BizSortVO();
            ls.setObjId(Long.valueOf((Integer) map.get("obj_id")));
            ls.setObjName((String) map.get("obj_name"));
            ls.setBizType((String) map.get("obj_type"));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });

//        for (int i = 0; i < 10; i++) {
//            BizSortVO vo = new BizSortVO(true);
//            list.add(vo);
//        }
        list.sort(Comparator.comparing(BizSortVO::getNum).reversed());
        List<BizSortVO> top10List = list.stream().limit(10).sorted(Comparator.comparing(BizSortVO::getNum)).collect(Collectors.toList());
        return AjaxResult.success(top10List);
    }

    @ApiOperation("系统压力排名")
    @GetMapping("/list/bizAccess")
    public AjaxResult listBizAccess(@ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
        List<BizAccessVO> list = new ArrayList<>();
        String sql = "SELECT bmdetail.model_id,subq.metrics_code,IFNULL(SUM(bmd.D_VALUE), 0) AS total_value " +
                "FROM biz_obj_metrics_data bmd " +
                "JOIN (" +
                "    SELECT bom.obj_metrics_id, bom.obj_id,md.metrics_code" +
                "    FROM biz_obj_metrics bom" +
                "    JOIN metrics_def md ON bom.metrics_id = md.metrics_id" +
                "    WHERE bom.metrics_name LIKE 'SPRING_BOOT/%' AND (md.metrics_code = 'pp.error' OR md.metrics_code='pp.tot')" +
                ") subq ON bmd.obj_metrics_id = subq.obj_metrics_id" +
                " JOIN biz_model_detail bmdetail ON subq.obj_id = bmdetail.obj_id" +
                " WHERE bmd.CREATE_TIME BETWEEN ? AND ?" +
                " GROUP BY bmdetail.model_id,subq.metrics_code";
        List<Map<String, Object>> acList = jdbcTemplate.queryForList(sql, start, end + " 23:59:59");
        Map<Integer, List<Map<String, Object>>> resultMap = acList.stream().collect(Collectors.groupingBy(mp -> (Integer) mp.get("model_id")));

        List<BizModel> modelList = bizModelService.selectBizModelList(new BizModel());
        for (BizModel model : modelList) {
            BizAccessVO vo = new BizAccessVO();
            Map<String, Object> map = new HashMap<>(16);
            vo.setModelId(model.getModelId());
            vo.setModelName(model.getModelName());
            map.put("name", model.getModelName());
            String sql1 = "SELECT COUNT(*) FROM alarm_record WHERE obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?) AND alarm_time BETWEEN ? AND ?";
            Long num = jdbcTemplate.queryForObject(sql1, Long.class, model.getModelId(), start, end + " 23:59:59");
            vo.setN1(num);

            sql1 = "SELECT COUNT(*) FROM biz_obj WHERE obj_type=1 AND obj_id IN(SELECT obj_id FROM biz_model_detail where model_id=?)";
            num = jdbcTemplate.queryForObject(sql1, Long.class, model.getModelId());
            vo.setN2(num);

            sql1 = "SELECT IFNULL(SUM(d_value),0) FROM biz_obj_metrics WHERE metrics_code='D_10163' AND obj_id IN(SELECT obj_id FROM biz_model_detail where model_id=?)";
            BigDecimal qdNum = jdbcTemplate.queryForObject(sql1, BigDecimal.class, model.getModelId());
            if (qdNum == null) {
                qdNum = BigDecimal.ZERO;
            }
            vo.setN3(qdNum.longValue());
            List<Map<String, Object>> vList = resultMap.get(model.getModelId().intValue());
            if (vList != null) {
                for (Map<String, Object> objectMap : vList) {
                    String code = (String) objectMap.get("metrics_code");
                    BigDecimal value = (BigDecimal) objectMap.get("total_value");
                    if ("pp.error".equals(code)) {
                        vo.setNum(value.longValue());
                    }
                    if ("pp.tot".equals(code)) {
                        vo.setAccess(value.longValue());
                    }
                }
            } else {
                vo.setNum(0L);
                vo.setAccess(0L);
            }

//            if ("D_10163".equals(code)) {
//                vo.setN3(value.longValue());
//            }

            if (vo.getN3() == null) {
                vo.setN3(0L);
            }
            list.add(vo);
        }


//        //还是得先用模拟数据。
//        String sql = "SELECT model_id,model_name FROM biz_model";
//        List<Map<String, Object>> mdList = jdbcTemplate.queryForList(sql);
//        mdList.forEach(map -> {
//            BizAccessVO vo = new BizAccessVO();
//            Integer modelId = (Integer) map.get("model_id");
//            String modelName = (String) map.get("model_name");
//            vo.setModelId(Long.valueOf(modelId));
//            vo.setModelName(modelName);
//            Vector<Object> v = new Vector<>();
//            String sql1 = "SELECT COUNT(*) FROM alarm_record WHERE obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?)";
//            v.add(modelId);
//            if (start != null && end != null) {
//                sql1 += " AND alarm_time BETWEEN ? AND ?";
//                v.add(start);
//                v.add(end + " 23:59:59");
//            }
//            Long num = jdbcTemplate.queryForObject(sql1, Long.class, v.toArray());
//            vo.setNum(num);
//            Vector<Object> v1 = new Vector<>();
//            String sql2 = "SELECT IFNULL(SUM(D_VALUE),0) FROM biz_obj_metrics_data " +
//                    "WHERE obj_metrics_id IN(" +
//                    "SELECT obj_metrics_id FROM biz_obj_metrics" +
//                    " WHERE metrics_id =(SELECT metrics_id FROM metrics_def WHERE metrics_code='pp.tot')" +
//                    " AND obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?))";
//            v1.add(modelId);
//            if (start != null && end != null) {
//                sql2 += " AND CREATE_TIME BETWEEN ? AND ?";
//                v1.add(start);
//                v1.add(end + " 23:59:59");
//            }
//            Long access = jdbcTemplate.queryForObject(sql2, Long.class, v1.toArray());
//            vo.setAccess(access);
//            vo.setN1(1L);
//
//            list.add(vo);
//        });
//        String[] names = {"市场服务", "市场出清", "市场结算", "市场合规", "信息发布"};
//        int[] as = new int[5];
//        int[] nums = new int[5];
//        for (int i = 0; i < as.length; i++) {
//            as[i] = new Random().nextInt(1000) + 200;
//            nums[i] = new Random().nextInt(500);
//        }
//        for (int i = 0; i < names.length; i++) {
//            BizAccessVO vo = new BizAccessVO();
//            vo.setModelId((long) (i + 1));
//            vo.setModelName(names[i]);
//            vo.setAccess((long) as[i]);
//            vo.setNum((long) nums[i]);
//            list.add(vo);
//        }

        list.sort(Comparator.comparing(BizAccessVO::getAccess).reversed());
        return AjaxResult.success(list);
    }

    @ApiOperation("组件告警数量排名")
    @GetMapping("/list/objAlarm")
    public AjaxResult objAlarm(@ApiParam(value = "对象ID") Long modelId, @ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {

        List<BizSortVO> list = new ArrayList();
        String sql = "SELECT a.obj_id,c.obj_name,COUNT(*) num FROM alarm_record a,biz_model_detail b,biz_obj c WHERE a.obj_id=b.obj_id AND b.obj_id=c.obj_id";
        Vector<Object> v = new Vector<>();
        if (modelId != null) {
            sql += " AND b.MODEL_ID=? ";
//            因为是模拟数据,所以这里要做个映射，之后要取消掉
//            int[] idRela = {4, 5, 15, 20, 23};
//            v.add(idRela[modelId.intValue() - 1]);
            v.add(modelId);
        }
        if (start != null && end != null) {
            sql += " AND a.alarm_time BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }
        sql += " GROUP BY a.obj_id,c.obj_name order by num desc";
        List<Map<String, Object>> results = jdbcTemplate.queryForList(sql, v.toArray());
        results.forEach(map -> {
            BizSortVO ls = new BizSortVO();
            ls.setObjId(Long.valueOf((Integer) map.get("obj_id")));
            ls.setObjName((String) map.get("obj_name"));
            ls.setNum((Long) map.get("num"));
            list.add(ls);
        });
        return AjaxResult.success(list);
    }

    @ApiOperation("组件压力排名")
    @GetMapping("/list/objAccess")
    public AjaxResult objAccess(@ApiParam(value = "对象ID") Long modelId, @ApiParam(value = "开始时间yyyy-mm-dd") String start, @ApiParam(value = "结束时间yyyy-mm-dd") String end) {
//        依然得用模拟数据
        Map<String, Object> result = new HashMap<>(16);
        List<BizAccessVO> list = alarmRecordService.objAccess(modelId, start, end);

        List<Map<String, Object>> dataList = new ArrayList<>();
        //获取趋势
        Vector<Object> v = new Vector<>();
        String totSql = "SELECT CREATE_TIME,SUM(D_VALUE) MAX_V,AVG(D_VALUE) AVG_V FROM biz_obj_metrics_data " +
                "WHERE obj_metrics_id IN(" +
                "SELECT obj_metrics_id FROM biz_obj_metrics" +
                " WHERE metrics_id =(SELECT metrics_id FROM metrics_def WHERE metrics_code='pp.tot')" +
                " AND obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?))";

        String errSql = "SELECT CREATE_TIME,SUM(D_VALUE) D_VALUE FROM biz_obj_metrics_data " +
                "WHERE obj_metrics_id IN(" +
                "SELECT obj_metrics_id FROM biz_obj_metrics" +
                " WHERE metrics_id =(SELECT metrics_id FROM metrics_def WHERE metrics_code='pp.error')" +
                " AND obj_id IN(SELECT obj_id FROM biz_model_detail WHERE model_id=?))";
        v.add(modelId);
        if (start != null && end != null) {
            totSql += " AND CREATE_TIME BETWEEN ? AND ?";
            errSql += " AND CREATE_TIME BETWEEN ? AND ?";
            v.add(start);
            v.add(end + " 23:59:59");
        }

        totSql += " group by create_time";
        errSql += " group by create_time";

        List<Map<String, Object>> totList = jdbcTemplate.queryForList(totSql, v.toArray());
        List<Map<String, Object>> errList = jdbcTemplate.queryForList(errSql, v.toArray());
        AtomicInteger index = new AtomicInteger();
        totList.forEach(map -> {
            Map<String, Object> data = new HashMap<>(16);
            LocalDateTime time = (LocalDateTime) map.get("CREATE_TIME");
            String formatTime = time.format(DateTimeFormatter.ofPattern("MM-dd HH:mm"));
            BigDecimal max = (BigDecimal) map.get("MAX_V");
            BigDecimal avg = (BigDecimal) map.get("AVG_V");
            data.put("time", formatTime);
            data.put("max", max);
            data.put("avg", avg.setScale(2, RoundingMode.HALF_UP));
            if (index.get() < errList.size()) {
                Map<String, Object> errMap = errList.get(index.get());
                BigDecimal error = (BigDecimal) errMap.get("D_VALUE");
                data.put("error", error);
            }
            dataList.add(data);
            index.getAndIncrement();
        });


        list.sort(Comparator.comparing(BizAccessVO::getAccess).reversed());
        result.put("objs", list);
        result.put("datas", dataList);
        return AjaxResult.success(result);
    }

    @ApiOperation("等级级别排名")
    @GetMapping("/list/levelSort")
    public AjaxResult listLevelSort(AlarmRecord alarmRecord) {
        List<LevelSortVO> list = new ArrayList();
        List<Map<String, Object>> results = alarmRecordService.levelSort(alarmRecord);
        results.forEach(map -> {
            LevelSortVO ls = new LevelSortVO();
            ls.setLevel((String) map.get("ALARM_LEVEL"));
            switch (ls.getLevel()) {
                case "low":
                    ls.setName("低");
                    break;
                case "mid":
                    ls.setName("中");
                    break;
                case "high":
                    ls.setName("高");
                    break;
                default:
            }
            ls.setNum((Long) map.get("NUM"));
            list.add(ls);
        });
        return AjaxResult.success(list);
    }

    @ApiOperation("按小时统计趋势")
    @GetMapping("/list/hours")
    public AjaxResult listHours(AlarmRecord alarmRecord) {
        List<Map<String, Object>> results = alarmRecordService.listHours(alarmRecord);
        return AjaxResult.success(results);
    }

    @ApiOperation("按指标统计")
    @GetMapping("/calc/ms")
    public AjaxResult calcMs(AlarmRecord alarmRecord) {
        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
        List<Map<String, Object>> hours = alarmRecordService.listHours(alarmRecord);
        Map<String, Object> map = new HashMap<>();
        map.put("detail", list);
        map.put("hour", hours);
        return AjaxResult.success(map);
    }

//    /**
//     * 导出告警记录列表
//     */
//    @ApiOperation("导出告警记录列表")
//    //@PreAuthorize("@ss.hasPermi('alarm:record:export')")
//    @Log(title = "告警记录", businessType = BusinessType.EXPORT)
//    @PostMapping("/export")
//    public void export(HttpServletResponse response, AlarmRecord alarmRecord) {
//        List<AlarmRecord> list = alarmRecordService.selectAlarmRecordList(alarmRecord);
//        ExcelUtil<AlarmRecord> util = new ExcelUtil<AlarmRecord>(AlarmRecord.class);
//        util.exportExcel(response, list, "告警记录数据");
//    }
//
//    /**
//     * 获取告警记录详细信息
//     */
//    @ApiOperation("获取告警记录详细信息")
//    //@PreAuthorize("@ss.hasPermi('alarm:record:query')")
//    @GetMapping(value = "/{alarmId}")
//    public AjaxResult getInfo(@PathVariable("alarmId") Long alarmId) {
//        return success(alarmRecordService.selectAlarmRecordByAlarmId(alarmId));
//    }
//
//    /**
//     * 新增告警记录
//     */
//    @ApiOperation("新增告警记录")
//    //@PreAuthorize("@ss.hasPermi('alarm:record:add')")
//    @Log(title = "告警记录", businessType = BusinessType.INSERT)
//    @PostMapping
//    public AjaxResult add(@RequestBody AlarmRecord alarmRecord) {
//        return toAjax(alarmRecordService.insertAlarmRecord(alarmRecord));
//    }
//
//    /**
//     * 修改告警记录
//     */
//    @ApiOperation("修改告警记录")
//    //@PreAuthorize("@ss.hasPermi('alarm:record:edit')")
//    @Log(title = "告警记录", businessType = BusinessType.UPDATE)
//    @PostMapping("/edit")
//    public AjaxResult edit(@RequestBody AlarmRecord alarmRecord) {
//        return toAjax(alarmRecordService.updateAlarmRecord(alarmRecord));
//    }
//
//    /**
//     * 删除告警记录
//     */
//    @ApiOperation("删除告警记录")
//    //@PreAuthorize("@ss.hasPermi('alarm:record:remove')")
//    @Log(title = "告警记录", businessType = BusinessType.DELETE)
//    @GetMapping("/del/{alarmIds}")
//    public AjaxResult remove(@PathVariable Long[] alarmIds) {
//        return toAjax(alarmRecordService.deleteAlarmRecordByAlarmIds(alarmIds));
//    }
}
