package com.jjt.biz.controller;

import com.jjt.biz.domain.BizObj;
import com.jjt.biz.domain.BizObjMetrics;
import com.jjt.biz.domain.MetricsDef;
import com.jjt.biz.domain.MetricsTpl;
import com.jjt.biz.service.*;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.SecurityUtils;
import com.jjt.common.utils.StringUtils;
import com.jjt.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 业务对象Controller
 *
 * @author jjt
 * @date 2024-08-09
 */
@Api(tags = "业务对象")
@RestController
@RequestMapping("/obj/obj")
public class BizObjController extends BaseController {
    @Resource
    private IBizObjService bizObjService;
    @Resource
    private IBizObjTplService bizObjTplService;
    @Resource
    private IBizObjMetricsService metricsService;
    @Resource
    private IBizObjPpService ppService;
    @Resource
    private IBizObjTplService tplService;
    @Resource
    private IBizObjMcService objMcService;
    @Resource
    private IPrometheusService prometheusService;

    @ApiOperation("选择模板")
    @GetMapping("/tpl/select/{objId}")
    public TableDataInfo select(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId) {
        startPage();
        List<MetricsTpl> list = bizObjTplService.selectTplList(objId);
        return getDataTable(list);
    }

    @ApiOperation("添加模板")
    @GetMapping("/tpl/add/{objId}")
    public AjaxResult add(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId, @ApiParam(value = "指标ID数组", required = true) Long[] tplIds) {
        return success(bizObjTplService.insertTplDetails(objId, tplIds));
    }

    @ApiOperation("删除模板")
    @GetMapping("/tpl/del/{objId}/{tplId}")
    public AjaxResult del(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId, @ApiParam(value = "模板ID", required = true) @PathVariable("tplId") Long tplId) {
        return success(bizObjTplService.del(objId, tplId));
    }

    @ApiOperation("选择指标")
    @GetMapping("/metrics/select/{objId}")
    public TableDataInfo metricsSelect(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId) {
        startPage();
        //只能选择prometheus的指标
        List<MetricsDef> list = metricsService.selectMetricsDefList(objId);
        return getDataTable(list);
    }

    @ApiOperation("公式验证")
    @GetMapping("/validate")
    public AjaxResult validate(@ApiParam(value = "公式内容", required = true) String exp) {
        //TODO 开发时，返回验证通过
//        System.err.println(exp);
//        DecimalFormat df = new DecimalFormat("#0.00");
//        Random r = new Random();
//        float f = r.nextFloat() * 100;
//        return AjaxResult.success(Float.parseFloat(df.format(f)));
        Float value = prometheusService.query(exp);
        if (value != null) {
            return success(value);
        } else {
            return error();
        }
    }

    @ApiOperation("数据接口配置进度")
    @GetMapping("/schedule/{objId}")
    public AjaxResult schedule(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId) {
        BizObjMetrics om = new BizObjMetrics();
        om.setObjId(objId);
        List<BizObjMetrics> allList = metricsService.selectBizObjMetricsList(om);

        List<BizObjMetrics> completeList = allList.stream().filter(oom -> StringUtils.isNotEmpty(oom.getDataExp())).collect(Collectors.toList());

        Map<String, Object> map = new HashMap<>(16);
        map.put("complete", completeList.size());
        map.put("total", allList.size());
        return AjaxResult.success(map);
    }

    @ApiOperation("添加指标")
    @GetMapping("/metrics/add/{objId}")
    public AjaxResult metricsAdd(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId, @ApiParam(value = "指标ID数组", required = true) Long[] metricsIds) {
        return success(metricsService.insertMetricsDetails(objId, metricsIds));
    }

    @ApiOperation("完成自定义指标配置")
    @GetMapping("/metrics/conf/{objId}")
    public AjaxResult metricsConf(@ApiParam(value = "对象ID", required = true) @PathVariable("objId") Long objId) {
        boolean flag = metricsService.insertMc(objId);
        if (flag) {
            return success();
        } else {
            return error("指标未配置完成");
        }
    }

    /**
     * 查询业务对象列表
     */
    @ApiOperation("查询业务对象列表")
    //@PreAuthorize("@ss.hasPermi('obj:obj:list')")
    @GetMapping("/list")
    public TableDataInfo list(BizObj bizObj) {
        startPage();
        List<BizObj> list = bizObjService.selectBizObjList(bizObj);
        return getDataTable(list);
    }

    /**
     * 导出业务对象列表
     */
    @ApiOperation("导出业务对象列表")
    //@PreAuthorize("@ss.hasPermi('obj:obj:export')")
    @Log(title = "业务对象", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizObj bizObj) {
        List<BizObj> list = bizObjService.selectBizObjList(bizObj);
        ExcelUtil<BizObj> util = new ExcelUtil<BizObj>(BizObj.class);
        util.exportExcel(response, list, "业务对象数据");
    }

    /**
     * 获取业务对象详细信息
     */
    @ApiOperation("获取业务对象详细信息")
    //@PreAuthorize("@ss.hasPermi('obj:obj:query')")
    @GetMapping(value = "/{objId}")
    public AjaxResult getInfo(@PathVariable("objId") Long objId) {
        return success(bizObjService.selectBizObjByObjId(objId));
    }

    /**
     * 新增业务对象
     */
    @ApiOperation("新增业务对象")
    //@PreAuthorize("@ss.hasPermi('obj:obj:add')")
    @Log(title = "业务对象", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizObj bizObj) {
        bizObj.setCreateBy("admin");
        bizObjService.insertBizObj(bizObj);
        return AjaxResult.success(bizObj);
    }

    /**
     * 修改业务对象
     */
    @ApiOperation("修改业务对象")
//    @PreAuthorize("@ss.hasPermi('obj:obj:edit')")
    @Log(title = "业务对象", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
   public AjaxResult edit(@RequestBody BizObj bizObj) {
        bizObj.setUpdateBy("admin");
        return toAjax(bizObjService.updateBizObj(bizObj));
    }

    /**
     * 删除业务对象
     */
    @ApiOperation("删除业务对象")
    //@PreAuthorize("@ss.hasPermi('obj:obj:remove')")
    @Log(title = "业务对象", businessType = BusinessType.DELETE)
    @GetMapping("/del/{objIds}")
    public AjaxResult remove(@PathVariable Long[] objIds) {

        return toAjax(bizObjService.deleteBizObjByObjIds(objIds));
    }
}
