package com.jjt.biz.controller;

import com.jjt.biz.domain.MetricsType;
import com.jjt.biz.service.IMetricsTypeService;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.SecurityUtils;
import com.jjt.common.utils.poi.ExcelUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 指标分类Controller
 *
 * @author jjt
 * @date 2024-08-08
 */
@Api(tags = "指标分类")
@RestController
@RequestMapping("/metrics/type")
public class MetricsTypeController extends BaseController {
    @Resource
    private IMetricsTypeService metricsTypeService;

    /**
     * 查询指标分类列表
     */
    @ApiOperation("查询指标分类列表")
    //@PreAuthorize("@ss.hasPermi('metrics:type:list')")
    @GetMapping("/list")
    public TableDataInfo list(MetricsType metricsType) {
        startPage();
        List<MetricsType> list = metricsTypeService.selectMetricsTypeList(metricsType);
        return getDataTable(list);
    }

    /**
     * 导出指标分类列表
     */
    @ApiOperation("导出指标分类列表")
    //@PreAuthorize("@ss.hasPermi('metrics:type:export')")
    @Log(title = "指标分类" , businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MetricsType metricsType) {
        List<MetricsType> list = metricsTypeService.selectMetricsTypeList(metricsType);
        ExcelUtil<MetricsType> util = new ExcelUtil<MetricsType>(MetricsType.class);
        util.exportExcel(response, list, "指标分类数据");
    }

    /**
     * 获取指标分类详细信息
     */
    @ApiOperation("获取指标分类详细信息")
    //@PreAuthorize("@ss.hasPermi('metrics:type:query')")
    @GetMapping(value = "/{typeId}")
    public AjaxResult getInfo(@PathVariable("typeId") Long typeId) {
        return success(metricsTypeService.selectMetricsTypeByTypeId(typeId));
    }

    /**
     * 新增指标分类
     */
    @ApiOperation("新增指标分类")
    //@PreAuthorize("@ss.hasPermi('metrics:type:add')")
    @Log(title = "指标分类" , businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody MetricsType metricsType) {
        metricsType.setCreateBy("admin");
        return toAjax(metricsTypeService.insertMetricsType(metricsType));
    }

    /**
     * 修改指标分类
     */
    @ApiOperation("修改指标分类")
    //@PreAuthorize("@ss.hasPermi('metrics:type:edit')")
    @Log(title = "指标分类" , businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
   public AjaxResult edit(@RequestBody MetricsType metricsType) {
        metricsType.setUpdateBy("admin");
        return toAjax(metricsTypeService.updateMetricsType(metricsType));
    }

    /**
     * 删除指标分类
     */
    @ApiOperation("删除指标分类")
    //@PreAuthorize("@ss.hasPermi('metrics:type:remove')")
    @Log(title = "指标分类" , businessType = BusinessType.DELETE)
    @GetMapping("/del/{typeIds}")
    public AjaxResult remove(@PathVariable Long[] typeIds) {
        return toAjax(metricsTypeService.deleteMetricsTypeByTypeIds(typeIds));
    }
}
