package com.jjt.push.controller;

import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.push.domain.PushRecord;
import com.jjt.push.service.IPushRecordService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 推送记录Controller
 *
 * @author jjt
 * @date 2024-10-11
 */
@Api(tags = "推送记录")
@RestController
@RequestMapping("/push/pushRecord")
public class PushRecordController extends BaseController {
    @Resource
    private IPushRecordService pushRecordService;

    /**
     * 查询推送记录列表
     */
    @ApiOperation("查询推送记录列表")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:list')")
    @GetMapping("/list")
    public TableDataInfo list(PushRecord pushRecord) {
        startPage();
        List<PushRecord> list = pushRecordService.selectPushRecordList(pushRecord);
        return getDataTable(list);
    }

    /**
     * 导出推送记录列表
     */
    @ApiOperation("导出推送记录列表")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:export')")
    @Log(title = "推送记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, PushRecord pushRecord) {
        List<PushRecord> list = pushRecordService.selectPushRecordList(pushRecord);
        ExcelUtil<PushRecord> util = new ExcelUtil<PushRecord>(PushRecord.class);
        util.exportExcel(response, list, "推送记录数据");
    }

    /**
     * 获取推送记录详细信息
     */
    @ApiOperation("获取推送记录详细信息")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:query')")
    @GetMapping(value = "/{prId}")
    public AjaxResult getInfo(@PathVariable("prId") Long prId) {
        return success(pushRecordService.selectPushRecordByPrId(prId));
    }

    /**
     * 新增推送记录
     */
    @ApiOperation("新增推送记录")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:add')")
    @Log(title = "推送记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody PushRecord pushRecord) {
        return toAjax(pushRecordService.insertPushRecord(pushRecord));
    }

    /**
     * 修改推送记录
     */
    @ApiOperation("修改推送记录")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:edit')")
    @Log(title = "推送记录", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
   public AjaxResult edit(@RequestBody PushRecord pushRecord) {
        return toAjax(pushRecordService.updatePushRecord(pushRecord));
    }

    /**
     * 删除推送记录
     */
    @ApiOperation("删除推送记录")
    //@PreAuthorize("@ss.hasPermi('push:pushRecord:remove')")
    @Log(title = "推送记录", businessType = BusinessType.DELETE)
    @GetMapping("/del/{prIds}")
    public AjaxResult remove(@PathVariable Long[] prIds) {
        return toAjax(pushRecordService.deletePushRecordByPrIds(prIds));
    }
}
