package com.jjt.risk.controller;

import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.risk.domain.RiskMsConfig;
import com.jjt.risk.service.IRiskMsConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 风险管理指标配置Controller
 *
 * @author jjt
 * @date 2024-09-12
 */
@Api(tags = "风险管理指标配置")
@RestController
@RequestMapping("/risk/config")
public class RiskMsConfigController extends BaseController {
    @Resource
    private IRiskMsConfigService riskMsConfigService;

    /**
     * 查询风险管理指标配置列表
     */
    @ApiOperation("查询风险管理指标配置列表")
    //@PreAuthorize("@ss.hasPermi('risk:config:list')")
    @GetMapping("/list")
    public TableDataInfo list(RiskMsConfig riskMsConfig) {
        startPage();
        List<RiskMsConfig> list = riskMsConfigService.selectRiskMsConfigList(riskMsConfig);
        return getDataTable(list);
    }

    /**
     * 导出风险管理指标配置列表
     */
    @ApiOperation("导出风险管理指标配置列表")
    //@PreAuthorize("@ss.hasPermi('risk:config:export')")
    @Log(title = "风险管理指标配置", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RiskMsConfig riskMsConfig) {
        List<RiskMsConfig> list = riskMsConfigService.selectRiskMsConfigList(riskMsConfig);
        ExcelUtil<RiskMsConfig> util = new ExcelUtil<RiskMsConfig>(RiskMsConfig.class);
        util.exportExcel(response, list, "风险管理指标配置数据");
    }

    /**
     * 获取风险管理指标配置详细信息
     */
    @ApiOperation("获取风险管理指标配置详细信息")
    //@PreAuthorize("@ss.hasPermi('risk:config:query')")
    @GetMapping(value = "/{hcId}")
    public AjaxResult getInfo(@PathVariable("hcId") Long hcId) {
        return success(riskMsConfigService.selectRiskMsConfigByHcId(hcId));
    }

    /**
     * 新增风险管理指标配置
     */
    @ApiOperation("新增风险管理指标配置")
    //@PreAuthorize("@ss.hasPermi('risk:config:add')")
    @Log(title = "风险管理指标配置", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RiskMsConfig riskMsConfig) {
        return toAjax(riskMsConfigService.insertRiskMsConfig(riskMsConfig));
    }

    /**
     * 修改风险管理指标配置
     */
    @ApiOperation("修改风险管理指标配置")
    //@PreAuthorize("@ss.hasPermi('risk:config:edit')")
    @Log(title = "风险管理指标配置", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
   public AjaxResult edit(@RequestBody RiskMsConfig riskMsConfig) {
        return toAjax(riskMsConfigService.updateRiskMsConfig(riskMsConfig));
    }

    /**
     * 删除风险管理指标配置
     */
    @ApiOperation("删除风险管理指标配置")
    //@PreAuthorize("@ss.hasPermi('risk:config:remove')")
    @Log(title = "风险管理指标配置", businessType = BusinessType.DELETE)
    @GetMapping("/del/{hcIds}")
    public AjaxResult remove(@PathVariable Long[] hcIds) {
        return toAjax(riskMsConfigService.deleteRiskMsConfigByHcIds(hcIds));
    }
}
